Package: multimediate
Title: Causal Mediation Analysis in Presence of Multiple Mediators
        Uncausally Related
Version: 0.1.4
Authors@R: 
    c(person(given = "Allan",
             family = "Jerolon",
             role = c("aut")),
      person(given = "Arce",
             family = "Domingo-Relloso",
             role = c("aut")),       
      person(given = "Samara",
             family = "Kiihl",
             role = c("cre", "aut"),
             email = "samarak@unicamp.br"),
      person(given = "Maria",
             family = "Tellez-Plaza",
             role = c("aut")))       
Description: Estimates key quantities in causal mediation analysis - including 
    average causal mediation effects (indirect effects), average direct 
    effects, total effects, and proportions mediated - in the presence of 
    multiple uncausally related mediators. Methods are described by 
    Jérolon et al., (2021) <doi:10.1515/ijb-2019-0088> and extended to 
    accommodate survival outcomes as described by Domingo-Relloso et al., 
    (2024) <doi:10.1101/2024.02.16.24302923>.
License: GPL-3
Encoding: UTF-8
LazyData: true
URL: https://samarafk.github.io/multimediate/
Imports: rmutil, MASS, mvtnorm, stats, graphics, timereg
Suggests: knitr, rmarkdown
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-07-09 18:59:16 UTC; samara
Author: Allan Jerolon [aut],
  Arce Domingo-Relloso [aut],
  Samara Kiihl [cre, aut],
  Maria Tellez-Plaza [aut]
Maintainer: Samara Kiihl <samarak@unicamp.br>
Repository: CRAN
Date/Publication: 2025-07-14 17:20:02 UTC
Built: R 4.6.0; ; 2026-04-26 08:59:56 UTC; unix
