# multimediate

This package is based on the work presented in A. Jérolon et al., [Causal mediation analysis in presence of multiple mediators uncausally related](https://doi.org/10.1515/ijb-2019-0088).

The work allowing multiple mediation analyzes with a survival outcome was 
largely developed with Arce Domingo-Relloso. 
This work is presented in Domingo-Relloso et al., [Causal mediation for uncausally related mediators in the context of survival analysis](https://doi.org/10.1101/2024.02.16.24302923). 


## Installation

You can install the package from the repository (https://github.com/samarafk/multimediate):

```r
library(devtools)
install_github("samarafk/multimediate") 
```

Or you can install the released version from [CRAN](https://cran.r-project.org) with:

```r
install.packages("multimediate")
```

## Examples

Vignettes and examples of usage can be found at https://samarafk.github.io/multimediate/.
